Module map_with_span

Module map_with_span 

Map result of parser with the input span it parsed

Structs

PMapWithSpan
Combinator to map result of parser with the span successfully parsed with the parser.

Functions

pmap_with_span
Function to easily build new PMapWithSpan though see crate::parser::core::Parser::map_with_span for easiest usage as a combinator.