Modules for displaying and organizing errors
Structs
- Error
- A parsing error that includes one or more kinds of errors, the input span where it occurred, and the remaining parser state.
Enums
- Error
Kind - Represents the kind of error encountered during parsing.
- Token
Pattern - A pattern representing one or more tokens or a string, used for descriptive error messages.
Traits
- Error
Display - A trait for custom error display formatting, allowing types like u8 that need to be display in a special way to be diffentiated from things that implement display